/*
*  Agent Commands definition for Ihes.Agent.Voice
*  By Shikai Chen
*/

#pragma once

namespace ihes{ namespace agent { namespace voice{

#define VOICE_CMD_READIT      0x1
#define VOICE_CMD_CHGVOICE    0x2

#define VOICE_LENGTH_LIMIT    2000  //limited to 1000 characters


    enum VOICE_CMD_RESULT
    {
        VOICE_READ_OK       = 0,
        TEXT_LENGTH_EXCEED  = 0x80000001,
        UNEXPECTED_ERROR    = 0x80000002,
        PENDING_QUEUE_FULL  = 0x80000003,
    };
    /*  Voice Indication:

    */

    enum Lang_Id
    {
        LANG_CH = 0,
        LANG_EN = 1,
    };

    class Readit
    {
    public:
        u32 operator() (const char * text, int size = -1, char * addr = "localhost" , Lang_Id langID= LANG_CH , ihes::net::CmdSender * sender = NULL, FILE *verbosefd = NULL)
        {
            static const char * lang_speakers[] = {"ScanSoft Mei-Ling_Full_22kHz" , "Microsoft Sam" };
           
            ihes::net::CmdSender *localSender = NULL;
            u32 ans = UNEXPECTED_ERROR;
            if (sender == NULL)
            {
                localSender = new ihes::net::CmdSender();
                sender = localSender;
            }

            if (size==-1) size = strlen(text);

            char leadingbuffer[1024];
            sprintf(leadingbuffer, "<VOICE REQUIRED=\"NAME=%s\"/>", lang_speakers[langID]);

            int leading_size;
            leading_size = strlen(leadingbuffer);

            if (strlen(text) > VOICE_LENGTH_LIMIT ) size = VOICE_LENGTH_LIMIT;

            char * text_to_read_8 = new char[size+leading_size];

            if (text_to_read_8 == NULL)
            {
                if (verbosefd)
                    fprintf(verbosefd, "[Readit]Memory insufficient.\n");
                ans = -1;
                goto fin;
            }

            strncpy(text_to_read_8, leadingbuffer, leading_size);
            strncpy(text_to_read_8+leading_size, text, size);

            if (verbosefd) fprintf(verbosefd, "Connecting to the agent@%s\n",addr);

            if (sender->connectAgent( ihes::net::AGENT_VOICE, addr) == -1)
            {
                if (verbosefd)
                    fprintf(verbosefd, "[Readit]Cannot connect to the agent.\n");
                ans = -1;
                goto fin;
            }
            if (verbosefd) fprintf(verbosefd, "Sending text...\n");

            if (sender->send_cmd(VOICE_CMD_READIT,0,(unsigned char *)text_to_read_8,size+leading_size) == -1)
            {
                if (verbosefd)
                    fprintf(verbosefd, "[Readit]Cannot send the text to the agent, abort.\n");
                sender->close();
                ans = -1;
                goto fin;
            }
            u16 anscmd;
            u32 result;

            if (sender->recv_ans(anscmd,result) != -1)
            {
                if (result != VOICE_READ_OK)
                {
                    if (verbosefd){
                        fprintf(verbosefd, "[Readit]Agent cannot read the text, reason is:\n");

                        switch (result)
                        {
                        case TEXT_LENGTH_EXCEED:
                            fprintf(verbosefd, "TEXT_LENGTH_EXCEED\n");
                            break;
                        case UNEXPECTED_ERROR:
                            fprintf(verbosefd, "UNEXPECTED_ERROR\n");
                            break;
                        case PENDING_QUEUE_FULL:
                            fprintf(verbosefd, "PENDING_QUEUE_FULL\n");
                            break;
                        }
                    }
                    ans = result;
                }
                else
                {
                    if (verbosefd)
                        fprintf(verbosefd, "The text will be read out.\n");   
                }
            }else
            {
                ans = -1;
                if (verbosefd)
                        fprintf(verbosefd, "Agent didn't response.\n");   
            }

            sender->close();
 
            if (verbosefd) fprintf(verbosefd, "Disconnected\n"); 
fin:
            if (localSender) delete localSender;
            if (text_to_read_8) delete text_to_read_8;
            return ans;
        }
    };

}}}
