#ifndef WINVER				// Allow use of features specific to Windows XP or later.
#define WINVER 0x0501		// Change this to the appropriate value to target other versions of Windows.
#endif

#ifndef _WIN32_WINNT		// Allow use of features specific to Windows XP or later.                   
#define _WIN32_WINNT 0x0501	// Change this to the appropriate value to target other versions of Windows.
#endif						

#ifndef _WIN32_WINDOWS		// Allow use of features specific to Windows 98 or later.
#define _WIN32_WINDOWS 0x0410 // Change this to the appropriate value to target Windows Me or later.
#endif

#ifndef _WIN32_IE			// Allow use of features specific to IE 6.0 or later.
#define _WIN32_IE 0x0600	// Change this to the appropriate value to target other versions of IE.
#endif

#include <tchar.h>

#define POINTER_64 __ptr64
#include <windows.h>

#define _ATL_CSTRING_EXPLICIT_CONSTRUCTORS      // some CString constructors will be explicit


#include <atlbase.h>
#include <atlstr.h>

#include <sapi.h>

#pragma comment(lib,"ole32.lib")
#pragma comment(lib,"sapi.lib")
#pragma comment(lib,"ws2_32.lib")

#include <stdio.h>
#include <iostream>
#include <process.h>

//begin IHES headers
#include "common/types.h"
#include "common/locker.h"
#include "common/net/netif.h"
#include "common/net/agentcmd.h"

#include "agent/voice/inc/voicecmd.h"

//STL
#include <queue>

using namespace std;

namespace ihes{ namespace agent{ namespace voice{

class Voice;

struct Connect_Info
{
    Voice * pThis;
    SOCKET  fd;

    Connect_Info(Voice * This, SOCKET tfd)
        : pThis(This)
        , fd(tfd){}
};

#define MAX_PENDING_CONNECTIONS  10
#define MAX_PENDING_QUEUE        20

class Voice
{
public:
    Voice()
        : _need_stop(false)
        , _pending_thread_count(0) 
        , _cmdhandler(ihes::net::AGENT_VOICE)
    {
        hEvent = CreateEvent( NULL, FALSE, FALSE, NULL);
        startTTSThread();
    }

    ~Voice()
    {
        stopTTSThread();
        CloseHandle(hEvent);
    }

    void   startlisten()
    {
        while(1)
        {
            SOCKET incoming_fd = _cmdhandler.listenNext();

            if (incoming_fd != -1)
            {
                if (_pending_thread_count >= MAX_PENDING_CONNECTIONS)
                {
                    closesocket(incoming_fd);
                    continue;
                }

                _counter_locker.lock();
                _pending_thread_count++;
                _counter_locker.unlock();

                _beginthread(Voice::connection_thread, 0, new Connect_Info(this, incoming_fd));
            }
        }
    }

protected:

    bool    startTTSThread()
    {
        _need_stop = false;
        _beginthread( Voice::tts_thread, 0,this);
        return true;
    }

    void    stopTTSThread()
    {
        _need_stop = true;
        SetEvent(hEvent);
    }

    static  void    tts_thread( void * pThis )
    {
        Voice * This = reinterpret_cast<Voice *>(pThis);

        This->_thread_ttsengine();
    }

    static  void    connection_thread( void * pThis)
    {
        Connect_Info * info = reinterpret_cast<Connect_Info *>(pThis);
        info->pThis->_thread_connection( info->fd );
        delete info;
    }

    void    _thread_ttsengine()
    {
        CComPtr<ISpVoice>   obj_ispvoice;
        HRESULT hr;

        CoInitializeEx(NULL,NULL);
        if (FAILED(::CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&obj_ispvoice)) || (obj_ispvoice==NULL) )
        {
            _tprintf(_T("ERROR, create IID_ISpVoice failed.\n"));
            
        }
        
        while(!_need_stop)
        {
            bool isQueueEmpty;
            _queue_locker.lock();
            isQueueEmpty = _text_queue.empty();
            _queue_locker.unlock();

            while (isQueueEmpty)
            {

                WaitForSingleObject( hEvent, INFINITE);
                if (_need_stop) break;
                _queue_locker.lock();
                isQueueEmpty = _text_queue.empty();
                _queue_locker.unlock();
            }
            if (_need_stop) break;

            TCHAR * txt;

            _queue_locker.lock();
            txt = _text_queue.front();
            _text_queue.pop();
            _queue_locker.unlock();

            obj_ispvoice->Speak( txt, SPF_IS_XML, NULL);
            delete [] txt;

            
        }

        ::CoUninitialize();
    }

    void    _thread_connection(SOCKET fd)
    {
        u16 maincmd, subcmd;

        u_result speech_len = _cmdhandler.recv_cmd( fd, maincmd, subcmd );
        
        char * pUTF8Buffer = NULL;
        WCHAR * pUTF16Buffer = NULL;
        
        u32     ans = UNEXPECTED_ERROR;

        switch (maincmd){
            case VOICE_CMD_READIT:
                int queue_size;
                _queue_locker.lock();
                queue_size = _text_queue.size();
                _queue_locker.unlock();

                if (queue_size> MAX_PENDING_QUEUE)
                {
                    ans = PENDING_QUEUE_FULL;
                    goto fin;
                }
                if (speech_len > 0)
                {
                    if (speech_len > VOICE_LENGTH_LIMIT)
                    {
                        ans = TEXT_LENGTH_EXCEED;
                        goto fin;
                    }

                    pUTF8Buffer = new char[speech_len];
                    if (pUTF8Buffer == NULL)
                    {
                        ans = TEXT_LENGTH_EXCEED;
                        goto fin;
                    }
                    u_result recv_size = _cmdhandler.recv_remaining( fd, (unsigned char *)pUTF8Buffer,  speech_len);

                    if (recv_size != speech_len)
                    {
                        ans = UNEXPECTED_ERROR;
                        goto fin;
                    }
                    
                    int   required_buffer_size;
                    required_buffer_size = MultiByteToWideChar( CP_UTF8, NULL, pUTF8Buffer, speech_len, NULL, 0);

                    if (required_buffer_size<=0 )
                    {
                        ans = UNEXPECTED_ERROR;
                        goto fin;
                    }
                    
                    pUTF16Buffer = new WCHAR[required_buffer_size + 1];
                    if (pUTF16Buffer == NULL)
                    {
                        ans = TEXT_LENGTH_EXCEED;
                        goto fin;
                    }
                    required_buffer_size = MultiByteToWideChar( CP_UTF8, NULL, pUTF8Buffer, speech_len, pUTF16Buffer, required_buffer_size);
                    if (required_buffer_size <=0 )
                    {
                        delete [] pUTF16Buffer;
                        ans = UNEXPECTED_ERROR;
                        goto fin;
                    }

                    pUTF16Buffer[required_buffer_size] =0;  //make sure the string is null-terminated.

                    //add it to the queue

                    _queue_locker.lock();
                    _text_queue.push(pUTF16Buffer);

                    _queue_locker.unlock();

        
                    //notify the tts thread to continue
                    SetEvent(hEvent);            
                    ans = VOICE_READ_OK;

                }else
                {
                    ans = VOICE_READ_OK;
                }
fin:
                if (pUTF8Buffer)
                    delete [] pUTF8Buffer;

                _cmdhandler.send_ans( fd, maincmd, ans);
                break;

            case VOICE_CMD_CHGVOICE:
                break;

            default:
                _cmdhandler.send_ans( fd, maincmd, UNEXPECTED_ERROR);
                break;
        }

        closesocket(fd);

        _counter_locker.lock();
        _pending_thread_count--;
        _counter_locker.unlock();
    }

    Locker  _queue_locker;
    Locker  _counter_locker;

    HANDLE  hEvent;
  
    int     _pending_thread_count;
    queue< WCHAR * >    _text_queue;

    bool    volatile    _need_stop;
    
    ihes::net::CmdHandler   _cmdhandler;
    
};

}}}

using namespace ihes::net;
using namespace ihes::agent::voice;

int _tmain(int argc, _TCHAR* argv[])
{
    WSADATA wsaData;
    int iResult = WSAStartup( MAKEWORD(2,2), &wsaData );
    if( iResult != NO_ERROR ){
        cerr<<"WSAStartup Failed"<<endl;
        return -1;
    }

    Voice theVoice;

    theVoice.startlisten();

    WSACleanup();

    return 0;
}