/*
 *      IHES Agent Communication Packet Interface
 *      By Shikai Chen
 *
 */


#pragma once

namespace ihes{namespace net{

#define IHES_PKT_MAGIC          0x53454849  /*IHES*/
#define PKT_DATA_SIZEMAX        512

//MAX Buffer length for receving dest socket addr
#define MAX_ADDR_LEN    256

//PORTS ALLOCATION
#define SERVER_EVENT_PORT       8100

//agent port is based on TCP
#define AGENT_PORT_BASE         4100

#define AGENT_PORT(x) \
    (AGENT_PORT_BASE + x)

#define GET_AGENT(x)  \
    ( x - AGENT_PORT_BASE )
//

enum   PKT_TYPES
{
  PKT_TYPE_PING                 = 0,
  PKT_TYPE_SENSOR_EVENT         = 1,
  PKT_TYPE_SEVER_EVENT          = 2,
  PKT_TYPE_CMD                  = 3,
  PKT_TYPE_ANS                  = 4,
};

#ifdef _WIN32
#pragma pack(1)
#endif

//GENERAL IHES PKT HEARDER
struct IHES_PKT_HEADER
{
  u32     magicnum;     /* = IHES_PKT_MAGI    */
  u16     pkt_type;     /* types of PKT_TYPES */
  u16     pkt_flg;          /* reserved */
  u32     data_len;

  
  IHES_PKT_HEADER(
      u32 len,
      u16 type = PKT_TYPE_PING,
      u16 flg  = 0
      )
      : magicnum(IHES_PKT_MAGIC)
      , pkt_type(type)
      , pkt_flg(flg)
      , data_len(len)
  {}

  bool  isValid()
  {
        return (magicnum == IHES_PKT_MAGIC);
  }
} __attribute__((packed));

struct PKT_PING
{
  IHES_PKT_HEADER       header;

  //TODO: add the flg meaning here
  PKT_PING( u32 len, u16 flg = 0)
    : header(len, PKT_TYPE_PING, flg)
  {}
} __attribute__((packed));

enum AGENT_TYPES
{
  SENSOR_UPS    =       1,
  AGENT_VOICE   =       2,
};




struct PKT_SENSOR_E
{
  IHES_PKT_HEADER       header;

  u16   sensor_type;    /* types of AGENT_TYPES */
  u16   event_info;

  PKT_SENSOR_E ( u32 len, u16 sensor_t = 0, u16 e_info = 0)
    : header( 0, PKT_TYPE_SENSOR_EVENT, 0)
    , sensor_type( sensor_t )
    , event_info ( e_info )
  {
    setDataSize( len );
  }
  //set the data size of this pkt, calc the total pkt's size and return it.
  u32 setDataSize( u32 datasize )
  {
    header.data_len = datasize + sizeof( PKT_SENSOR_E);
    return header.data_len;
  }

  u32 getDataSize ()
  {
    return header.data_len - sizeof( PKT_SENSOR_E );
  }
} __attribute__((packed));


struct PKT_AGENT_CMD
{
  IHES_PKT_HEADER       header;
  u16                   cmd;
  u16                   subcmd;
  
  PKT_AGENT_CMD ( u32 len, u16 main_cmd = 0, u16 sub_cmd = 0)
    : header( 0, PKT_TYPE_CMD, 0)
    , cmd( main_cmd )
    , subcmd ( sub_cmd )
  {
    setDataSize( len );
  }
  //set the data size of this pkt, calc the total pkt's size and return it.
  u32 setDataSize( u32 datasize )
  {
    header.data_len = datasize + sizeof( PKT_AGENT_CMD);
    return header.data_len;
  }

  u32 getDataSize ()
  {
    return header.data_len - sizeof( PKT_AGENT_CMD );
  }
} __attribute__((packed));

struct PKT_AGENT_ANS
{
  IHES_PKT_HEADER       header;
  u16                   agent_type;    /* types of AGENT_TYPES */
  u16                   cmd;
  u32                   result;
  
  PKT_AGENT_ANS ( u32 len, u16 agenttype , u16 ansofcmd = 0, u32 ansresult = 0)
    : header( 0, PKT_TYPE_ANS, 0)
    , cmd( ansofcmd )
    , result ( ansresult )
    , agent_type(agenttype)
  {
    setDataSize( len );
  }
  //set the data size of this pkt, calc the total pkt's size and return it.
  u32 setDataSize( u32 datasize )
  {
    header.data_len = datasize + sizeof( PKT_AGENT_ANS);
    return header.data_len;
  }

  u32 getDataSize ()
  {
    return header.data_len - sizeof( PKT_AGENT_ANS );
  }
} __attribute__((packed));

#ifdef _WIN32
#pragma pack()
#endif

}}
