#ifndef WINVER				// Allow use of features specific to Windows XP or later.
#define WINVER 0x0501		// Change this to the appropriate value to target other versions of Windows.
#endif

#ifndef _WIN32_WINNT		// Allow use of features specific to Windows XP or later.                   
#define _WIN32_WINNT 0x0501	// Change this to the appropriate value to target other versions of Windows.
#endif						

#ifndef _WIN32_WINDOWS		// Allow use of features specific to Windows 98 or later.
#define _WIN32_WINDOWS 0x0410 // Change this to the appropriate value to target Windows Me or later.
#endif

#ifndef _WIN32_IE			// Allow use of features specific to IE 6.0 or later.
#define _WIN32_IE 0x0600	// Change this to the appropriate value to target other versions of IE.
#endif


#include <tchar.h>

#define POINTER_64 __ptr64
#include <windows.h>

#pragma comment(lib,"ws2_32.lib")

#include <stdio.h>
#include <iostream>

//begin IHES headers
#include "common/types.h"
#include "common/locker.h"
#include "common/net/netif.h"
#include "common/net/agentcmd.h"

#include "agent/voice/inc/voicecmd.h"

using namespace std;

using namespace ihes::net;
using namespace ihes::agent::voice;

#define HOSTNAME_MAX    260

const TCHAR * support_langs[] = {_T("cn") , _T("en") };


void    printUsage()
{
    wcout<< L"USAGE: readit [-d server addr] [-v] [-l language] [-f filename] [Text to read]\n" 
         << L"       -d IHES.Agent.Voice agent's address\n"
         << L"          If not specified, the local agent will be used.\n"
         << L"       -v Verbose mode\n"
         << L"       -l {cn|en} Specify the speaker's language. Chinese will be selected by default.\n"
         << L"       -f filename Specify the file that will be read.\n"
         << L"*NOTE* if not specified the text to read, readit will read the user input from stdin."<<endl;
} 

int _tmain(int argc, _TCHAR* argv[])
{
    int current_arg;
    int ans = -1;
    bool isVerboseMode = false;
    TCHAR *server_addr=NULL, *text_to_read=NULL;
    TCHAR *lang = _T("cn");
    
    FILE *file2read = NULL;
//command line parsing
    for ( current_arg=1; current_arg<argc; current_arg++)
    {
        TCHAR *current = argv[current_arg];

        if ( _tcscmp( current, _T("-d")) == 0)
        {
            if (++current_arg <argc ){
                server_addr = argv[current_arg];
            }
            continue;
        }
        if ( _tcscmp( current, _T("-l")) == 0)
        {
            if (++current_arg <argc ){
                lang = argv[current_arg];
            }
            continue;
        }
        else if ( _tcscmp( current, _T("-h")) == 0)
        {
            printUsage();
            return 0;
        }
        else if ( _tcscmp( current, _T("-v")) == 0)
        {
            isVerboseMode = true;
        }
        else if ( _tcscmp( current, _T("-f")) == 0)
        {
            if (++current_arg <argc ){
                file2read = _wfopen(argv[current_arg],L"r");
               
                if (file2read == NULL)
                {
                    wcerr<<L"Cannot open the file:"<<argv[current_arg]<<endl;
                    return -1;
                }
            }
            continue;
        }
        else
        {
            text_to_read = argv[current_arg];
        }
    }

    if (text_to_read == NULL && file2read==NULL)
    {
        file2read = stdin;
        
    }

    if (file2read!=NULL)
    {
        char *readoutbuff_a = new char[VOICE_LENGTH_LIMIT];
        int buff_size = 0;
        
       

        buff_size = fread(readoutbuff_a, sizeof(char), VOICE_LENGTH_LIMIT-1, file2read);

        if (buff_size>=0)
        {
            readoutbuff_a[buff_size] = 0;
        }

        buff_size = MultiByteToWideChar( CP_ACP, NULL,readoutbuff_a ,-1, NULL, 0);
        text_to_read = NULL;
        if (buff_size <= 0) goto ENDUP;
        text_to_read = new TCHAR[buff_size];

        if (text_to_read==NULL) goto ENDUP;

        
        MultiByteToWideChar( CP_ACP, NULL,readoutbuff_a ,-1, text_to_read, buff_size);

        delete [] readoutbuff_a;

    }

    int selected_speaker_id = 0;
    for (; selected_speaker_id < _countof(support_langs); selected_speaker_id++)
    {
        if ( _tcscmp(support_langs[selected_speaker_id], lang ) == 0 ) break;
    }

    if (selected_speaker_id == _countof(support_langs))
    {
        wcout<<L"Cannot found a voice which speaks "<<lang<<endl;
        wcout<<L"Default voice will be used"<<endl;
        selected_speaker_id = 0;
    }

 
//codepage converting
    //convert the host name to ascii
    if (!server_addr) server_addr = L"localhost";
    char hostname_buffer[HOSTNAME_MAX];
    WideCharToMultiByte( CP_ACP, NULL, server_addr, -1, hostname_buffer, HOSTNAME_MAX, NULL, NULL);

    //conver the text
    if (_tcslen(text_to_read) > VOICE_LENGTH_LIMIT ) text_to_read[VOICE_LENGTH_LIMIT] = 0;

    int required_len = WideCharToMultiByte( CP_UTF8, NULL, text_to_read, -1, NULL, 0, NULL, NULL);

    if (required_len<0)
    {
        wcerr<<L"Covert encoding failed."<<endl;
        ans= -1;
        goto ENDUP;
    }

    char * text_to_read_8 = new char[required_len];
    if (text_to_read_8 == NULL)
    {
        wcerr<<L"Covert encoding failed. Memory insufficient."<<endl;
        ans= -1;
        goto ENDUP;
    }

    WideCharToMultiByte( CP_UTF8, NULL, text_to_read, -1, text_to_read_8, required_len, NULL, NULL);

    WSADATA wsaData;

    int iResult = WSAStartup( MAKEWORD(2,2), &wsaData );
    if( iResult != NO_ERROR ){
        wcerr<<L"WSAStartup Failed"<<endl;
        ans= -1;
        goto ENDUP;
    }
//TODO: write the indication parsing here

    ans = 0;
    Readit readit;
    readit(text_to_read_8, required_len, hostname_buffer, (Lang_Id)selected_speaker_id, NULL, (isVerboseMode)?stdout:NULL );
    WSACleanup();

ENDUP:
    if (file2read)
    {
        fclose(file2read);
        delete [] text_to_read;
    }
    return ans;
}