/*
 *  The readit tools for IHES
 *  Linux version
 *
 *  By Shikai Chen
 */

#include <iostream>
#include <unistd.h>
#include <netdb.h>
#include <errno.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <time.h>

#include "common/utilt.h"
#include "common/types.h"
#include "common/locker.h"
#include "common/net/netif.h"
#include "common/net/agentevent.h"
#include "common/net/agentcmd.h"

#include "agent/voice/inc/voicecmd.h"

using namespace std;

using namespace ihes::net;
using namespace ihes::agent::voice;


const char * support_langs[] = {"cn" , "en" };

void    printUsage()
{
    cout<< "USAGE: readit [-d server addr] [-v] [-l language] [-f filename] [Text to read]\n" 
         << "       -d IHES.Agent.Voice agent's address\n"
         << "          If not specified, the local agent will be used.\n"
         << "       -v Verbose mode\n"
         << "       -l {cn|en} Specify the speaker's language. Chinese will be selected by default.\n"
         << "       -f filename Specify the file that will be read.\n"
         << "*NOTE* if not specified the text to read, readit will read the user input from stdin."<<endl;
} 

int main(int argc, char* argv[])
{
    int current_arg;
    bool isVerboseMode = false;
    char *server_addr=NULL, *text_to_read=NULL;
    char *lang = "cn";
    FILE *file2read = NULL;

    int selected_speaker_id = 0;
    int ans = -1;
//command line parsing
    for ( current_arg=1; current_arg<argc; current_arg++)
    {
        char *current = argv[current_arg];

        if ( strcmp( current, "-d") == 0)
        {
            if (++current_arg <argc ){
                server_addr = argv[current_arg];
            }
            continue;
        }
        if ( strcmp( current, "-l") == 0)
        {
            if (++current_arg <argc ){
                lang = argv[current_arg];
            }
            continue;
        }
        else if ( strcmp( current, "-h") == 0)
        {
            printUsage();
            return 0;
        }
        else if ( strcmp( current, "-v") == 0)
        {
            isVerboseMode = true;
        }
        else if ( strcmp( current, "-f") == 0)
        {
            if (++current_arg <argc ){
                file2read = fopen(argv[current_arg],"r");
               
                if (file2read == NULL)
                {
                    cerr<<"Cannot open the file:"<<argv[current_arg]<<endl;
                    return -1;
                }
            }
            continue;
        }
        else
        {
            text_to_read = argv[current_arg];
        }
    }

    if (text_to_read == NULL && file2read==NULL)
    {
        file2read = stdin;
    }
    if (file2read!=NULL)
    {
        text_to_read = NULL;
        text_to_read = new char[VOICE_LENGTH_LIMIT];
        if (text_to_read==NULL) goto ENDUP;
        int buff_size = fread(text_to_read, sizeof(char), VOICE_LENGTH_LIMIT, file2read);

        if (buff_size>=0)
        {
            text_to_read[buff_size] = 0;
        }

    }
    for (; selected_speaker_id < _countof(support_langs); selected_speaker_id++)
    {
        if ( strcmp(support_langs[selected_speaker_id], lang ) == 0 ) break;
    }

    if (selected_speaker_id == _countof(support_langs))
    {
        cout<<"Cannot found a voice which speaks "<<lang<<endl;
        cout<<"Default voice will be used"<<endl;
        selected_speaker_id = 0;
    }

    if (!server_addr) server_addr = "localhost";
    Readit readit;
   
    ans= readit(text_to_read, strlen(text_to_read), server_addr, (Lang_Id)selected_speaker_id, NULL, (isVerboseMode)?stdout:NULL );
ENDUP:
    if (file2read)
    {
        fclose(file2read);
        delete [] text_to_read;
    }

    return ans;
}
